
c     Subroutines on this file:
c      nc_char_attrib   - writes one character netCDF attribute
c      nc_int_attrib    - writes one or more 4 byte integer netCDF
c                          attributes
c      nc_float_attrib  - writes one or more 4 byte floating point
c                          netCDF attributes
c      nc_miss_val_i2   - includes  the numeric attributes 'missing_value'
c                          of type integer*2 (short)
c      nc_miss_val_i4   - includes 'missing_value' of type integer
c      nc_miss_val_r    - includes 'missing_value' of type real (float)
c      nc_miss_val_d    - includes 'missing_value' of type double
c      nc_miss_val_warning
c                       - writes warning message(s) when attempting to
c                          define existing attribute 'missing_values'

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c     The subroutine "nc_char_attrib" writes
c      - the name of a character attribute ('name')
c      - the value of the character attribute ('attrib')
c     to the current netCDF header for the variable indicated by 'id'.
c     If an error occurs, the name of the variable ('vrbl') is passed
c     to an error message subroutines which terminates the program.

      subroutine nc_char_attrib(id, name, attrib, vrbl)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'

c     Input:
      integer id
      character*(*) name
      character*(*) attrib
      character*(*) vrbl

c     Local:
      integer nc_status,size_name,size_attrib,size_vrbl,no
      character*120 old_attrib

c     Function:
      integer str_len


      size_name = str_len(name)
      size_attrib = str_len(attrib)
      size_vrbl = str_len(vrbl)

c     Check existence:
      nc_status = NF_INQ_ATTID(nc_id, id, name(1:size_name), no)
      if (nc_status .eq. no_att_match) then

c        New attribute, add:
         nc_status = NF_PUT_ATT_TEXT(nc_id, id, name(1:size_name),
     >                               size_attrib, attrib(1:size_attrib))
         if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, name, vrbl, 1)

      else if (nc_status .eq. NF_NOERR) then

c        Existing attribute, warning mssg.(s):
         write(*,*)'  ... attribute "',name(1:size_name),
     >             '" of variable "',vrbl(1:size_vrbl),'" exists. ',
     >             'Continuing execution.'

         nc_status = NF_INQ_ATTLEN(nc_id, id, name(1:size_name), no)
         old_attrib = ''
         nc_status = NF_GET_ATT_TEXT(nc_id, id, name(1:size_name),
     >                               old_attrib)
         if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, name, vrbl, 2)

c         no = str_len(old_attrib)
         if (attrib(1:size_attrib) .ne. old_attrib(1:no)) then
            write(*,*)'       WARNING! Texts differ:'
            write(*,*)'            existing:  ',old_attrib(1:no)
            write(*,*)'            this spec: ',attrib(1:size_attrib)
         end if

      else

         call handle_err_txt(nc_status, name, vrbl, 3)

      end if

      return
      end



c     The subroutine "nc_int_attrib" writes
c      - the name of an integer (4 byte) attribute ('name')
c      - the value of the integer attribute(s) ('attrib(no)')
c     to the current netCDF header for the variable indicated by 'id'.
c     If an error occurs, the name of the variable ('vrbl') is passed
c     to an error message subroutines which terminates the program.

      subroutine nc_int_attrib(id, name, no, attrib, vrbl)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'

c     Parameter:
      real epsilon
      parameter (epsilon = 1.e-6)

c     Input:
      integer id,no,attrib(no)
      character*(*) name
      character*(*) vrbl

c     Local:
      integer nc_status,size_name,size_vrbl,old_attrib,attrib_no

c     Function:
      integer str_len


      size_name = str_len(name)
      size_vrbl = str_len(vrbl)

c     Check existence:
      nc_status = NF_INQ_ATTID(nc_id, id, name(1:size_name), attrib_no)

      if (nc_status .eq. no_att_match) then

c        New attribute, add:
         nc_status = NF_PUT_ATT_INT(nc_id, id, name(1:size_name),
     >                              NF_INT, no, attrib)
        if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, name, vrbl, 1)

      else if (nc_status .eq. NF_NOERR) then

c        Existing attribute, warning mssg.(s):
         write(*,*)'  ... attribute "',name(1:size_name),
     >             '" of variable "',vrbl(1:size_vrbl),'" exists. ',
     >             'Continuing execution.'

         if (no .eq. 1) then

c           Further testing/mssg.:

            nc_status = NF_GET_ATT_INT(nc_id, id, name(1:size_name),
     >                                 old_attrib)
            if (nc_status .ne. NF_NOERR)
     >         call handle_err_txt(nc_status, name, vrbl, 2)
            if (attrib(1) .ne. old_attrib) then
               write(*,*)'       WARNING! Real numbers differ:'
               write(*,*)'            existing:  ',old_attrib
               write(*,*)'            this spec: ',attrib(1)
            end if

         else

c           No further testing:
            write(*,*)'Values not checked for multiple entry arrays.'

         end if

      else

         call handle_err_txt(nc_status, name, vrbl, 3)

      end if

      return
      end



c     The subroutine "nc_float_attrib" writes
c      - the name of a float (4 byte) attribute ('name')
c      - the value of the float attribute(s) ('attrib(no)')
c     to the current netCDF header for the variable indicated by 'id'.
c     If an error occurs, the name of the variable ('vrbl') is passed
c     to an error message subroutines which terminates the program.

      subroutine nc_float_attrib(id, name, no, attrib, vrbl)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'

c     Parameter:
      real epsilon
      parameter (epsilon = 1.e-6)

c     Input:
      integer id,no
      real attrib(no)
      character*(*) name
      character*(*) vrbl

c     Local:
      integer nc_status,size_name,size_vrbl,n
      real ratio,old_attrib

c     Function:
      integer str_len


      size_name = str_len(name)
      size_vrbl = str_len(vrbl)

c     Check existence:
      nc_status = NF_INQ_ATTID(nc_id, id, name(1:size_name), n)
      if (nc_status .eq. no_att_match) then

c        New attribute, add:
         nc_status = NF_PUT_ATT_REAL(nc_id, id, name(1:size_name),
     >                               NF_FLOAT, no, attrib)
         if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, name, vrbl, 1)

      else if (nc_status .eq. NF_NOERR) then

c        Existing attribute, warning mssg.(s):
         write(*,*)'  ... attribute "',name(1:size_name),
     >             '" of variable "',vrbl(1:size_vrbl),'" exists. ',
     >             'Continuing execution.'

         if (no .eq. 1) then

c           Further testing/mssg.:

            nc_status = NF_GET_ATT_REAL(nc_id, id, name(1:size_name),
     >                                  old_attrib)
            if (nc_status .ne. NF_NOERR)
     >         call handle_err_txt(nc_status, name, vrbl, 2)
            if (old_attrib .ne. 0.) then
               ratio = attrib(1)/old_attrib
            else if (attrib(1) .eq. 0.) then
               ratio = 1.
            else
               ratio = 1.e10
            end if
            if (abs(ratio - 1.) .gt. epsilon) then
               write(*,*)'       WARNING! Real numbers differ:'
               write(*,*)'            existing:  ',old_attrib
               write(*,*)'            this spec: ',attrib(1)
            end if

         else

c           No further testing:
            write(*,*)'Values not checked for multiple entry arrays.'

         end if

      else

         call handle_err_txt(nc_status, name, vrbl, 3)

      end if

      return
      end



c     The subroutines "nc_miss_val_X" writes the attribute
c      - 'missing_value' (based on the contents of "missing_values.inc")
c     to the current netCDF header for the variable indicated by 'id'.
c     If an error occurs, the name of the variable ('vrbl') is passed
c     to an error message subroutines which terminates the program.

      subroutine nc_miss_val_i2(id, vrbl)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'
      include 'missing_values.inc'

c     Input:
      integer id
      character*(*) vrbl

c     Local:
      integer nc_status,no


c     Check existence:
      nc_status = NF_INQ_ATTID(nc_id, id, 'missing_value', no)
      if (nc_status .eq. no_att_match) then
c        New attribute, add:
         nc_status = NF_PUT_ATT_INT2(nc_id, id, 'missing_value',
     >                               NF_SHORT, 1, miss_val_i2)
         if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, 'missing_value', vrbl, 11)
      else if (nc_status .eq. NF_NOERR) then
c        Existing attribute, warning mssg.(s):
         call  nc_miss_val_warning(id, vrbl, NF_SHORT, 'NF_SHORT')
      else
         call handle_err_txt(nc_status, 'missing_value', vrbl, 12)
      end if

      return
      end



      subroutine nc_miss_val_i4(id, vrbl)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'
      include 'missing_values.inc'

c     Input:
      integer id
      character*(*) vrbl

c     Local:
      integer nc_status,no


c     Check existence:
      nc_status = NF_INQ_ATTID(nc_id, id, 'missing_value', no)
      if (nc_status .eq. no_att_match) then
c        New attribute, add:
         nc_status = NF_PUT_ATT_INT(nc_id, id, 'missing_value',
     >                              NF_INT, 1, miss_val_i4)
         if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, 'missing_value', vrbl, 21)
      else if (nc_status .eq. NF_NOERR) then
c        Existing attribute, warning mssg.(s):
         call  nc_miss_val_warning(id, vrbl, NF_INT, 'NF_INT')
      else
         call handle_err_txt(nc_status, 'missing_value', vrbl, 22)
      end if

      return
      end



      subroutine nc_miss_val_r(id, vrbl)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'
      include 'missing_values.inc'

c     Input:
      integer id
      character*(*) vrbl

c     Local:
      integer nc_status,no


c     Check existence:
      nc_status = NF_INQ_ATTID(nc_id, id, 'missing_value', no)
      if (nc_status .eq. no_att_match) then
c        New attribute, add:
         nc_status = NF_PUT_ATT_REAL(nc_id, id, 'missing_value',
     >                               NF_FLOAT, 1, miss_val_r)
         if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, 'missing_value', vrbl, 31)
      else if (nc_status .eq. NF_NOERR) then
c        Existing attribute, warning mssg.(s):
         call  nc_miss_val_warning(id, vrbl, NF_FLOAT, 'NF_FLOAT')
       else
         call handle_err_txt(nc_status, 'missing_value', vrbl, 32)
      end if

      return
      end



      subroutine nc_miss_val_d(id, vrbl)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'
      include 'missing_values.inc'

c     Input:
      integer id
      character*(*) vrbl

c     Local:
      integer nc_status,no


c     Check existence:
      nc_status = NF_INQ_ATTID(nc_id, id, 'missing_value', no)
      if (nc_status .eq. no_att_match) then
c        New attribute, add:
         nc_status = NF_PUT_ATT_DOUBLE(nc_id, id, 'missing_value',
     >                                NF_DOUBLE, 1, miss_val_d)
         if (nc_status .ne. NF_NOERR)
     >      call handle_err_txt(nc_status, 'missing_value', vrbl, 41)
      else if (nc_status .eq. NF_NOERR) then
c        Existing attribute, warning mssg.(s):
         call  nc_miss_val_warning(id, vrbl, NF_DOUBLE, 'NF_DOUBLE')
       else
         call handle_err_txt(nc_status, 'missing_value', vrbl, 42)
      end if

      return
      end



c     The subroutine "nc_miss_val_warning" produces warning messages
c     when the user tries to specify a "missing values" attribute for
c     a variable which already has such an attribute.

      subroutine nc_miss_val_warning(id, vrbl, type, type_text)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'

c     Input:
      integer id,type
      character*(*) vrbl
      character*(*) type_text

c     Local:
      integer nc_status,size,oldtype,type_text_size

c     Function:
      integer str_len


      size = str_len(vrbl)
      type_text_size = str_len(type_text)

c     Existing attribute, warning mssg.:
      write(*,*)'  ... attribute "missing_value"',
     >          ' of variable "',vrbl(1:size),'" exists. ',
     >          'Continuing execution.'

      nc_status = NF_INQ_ATTTYPE(nc_id, id, 'missing_value', oldtype)
      if (nc_status .ne. NF_NOERR)
     >   call handle_err_txt(nc_status, 'missing_value', vrbl, 101)

      if (oldtype .ne. type) then
c        Types differ, warning mssg.:
         write(*,*)'       WARNING! Types differ:'
         write(*,*)'            existing:  ',oldtype
         write(*,*)'            this spec: ',type,' (',
     >             type_text(1:type_text_size),')'
      end if

      return
      end


