
c     Subroutines on this file:
c      write_nc_dim_header
c                       - writes a header for a netCDF dimension
c      write_nc_var_header
c                       - writes a header for a netCDF variable
c      write_nc_time_header
c                       - writes a header for the netCDF variable 'time'

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc



c     ********************
c     General description:
c     *********************

c     The subroutine "write_nc_dim_header" includes a new dimension
c      on the file with netCDF ID 'nc_id', and prints the actual dimension
c      on the screen. Testing of prior existence is performed initially.


c     ***********************
c     Input/output variables:
c     ***********************

c      dim_name       (s)  - name of the dimension that will be included
c      dim_size       (i)  - size of dimension (e.g., 'nx')
c      dim_id         (i)  - output, netCDF dimension ID

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      s  - string           [ character*(*) ]


      subroutine write_nc_dim_header(dim_name,dim_size,dim_id)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'err_codes.inc'

c     Input:
      integer dim_size
      character*(*) dim_name

c     Output:
      integer dim_id

c     Local:
      integer size,old_dim_size,nc_status

c     Function:
      integer str_len


      size = str_len(dim_name)

c     Explore existence:
      nc_status = NF_INQ_DIMID(nc_id, dim_name(1:size), dim_id)

c     Define?
      if (nc_status .eq. no_dim_match) then

c        Define variable:
         nc_status = NF_DEF_DIM(nc_id, dim_name(1:size), dim_size,
     >                          dim_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,801)
         write(*,*)' ',dim_name(1:size),': ',dim_size

      else if (nc_status .eq. NF_NOERR) then

c        Warning message(s):
         write(*,*)'  ... dimension "',dim_name(1:size),'" previously ',
     >             'defined. Continuing execution.'
         nc_status = NF_INQ_DIMLEN(nc_id, dim_id, old_dim_size)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,802)
         if (dim_size .ne. old_dim_size .and.
     >       dim_size .ne. NF_UNLIMITED) then
            write(*,*)'       WARNING! Dimensions differ:'
            write(*,*)'            existing:  ',old_dim_size
            write(*,*)'            this spec: ',dim_size
         end if
      else
         call handle_err(nc_status,803)
      end if


      return
      end



c     ********************
c     General description:
c     *********************

c     The subroutine "write_nc_var_header" includes
c      * a new variable (of type SHORT, LONG, FLOAT or DOUBLE)
c          on the file with netCDF ID 'nc_id'
c      * a  string  attribute "long_name"    (from 'name.inc')
c      * a  string  attribute "units"        (from 'name.inc')
c      * an integer attribute "missing_value"
c      * a  float   attribute "scale_factor" (implicitely from 'specs.inc')
c      * a  float   attribute "add_offset"   (implicitely from 'specs.inc')
c     NOTE 1:
c      No attributes are written if 'extended' is 0 ("false") on input
c     NOTE 2:
c      If extended is non-zero ("true") and the variable is of type FLOAT
c      or DOUBLE, the attributes 'scale_factor' and 'add_offset" are
c      disregarded (i.e., not included on the header).


c     ***********************
c     Input/output variables:
c     ***********************

c      variable       (s)  - name of the variable that will be included,
c                             must match one of the names in 'var_name'
c                             on 'name.inc'
c      type           (i)  - type of integer variable
c                             (NF_SHORT, NF_INT, NF_FLOAT or NF_DOUBLE)
c      dim_options    (i)  - no. of sets of different (x,y,z,t)-dimensions
c      var_n_dim      (iA) - no. of dimensions for the various
c                             'dim_options' dimensional combinations
c      var_dim_ids    (iA) - netCDF IDs for the various dimensional
c                             combinations
c      var_id         (i)  - output, netCDF variable ID
c      extended       (i)  - switch for including attributes on header:
c                             0: no attributes
c                             1: include attributes
c      unit           (s)  - unit used for the present variable
c                             (generally, SI units are used)
c      offset         (rA) - value (single value array)
c                             for the attribute "add_offset"
c      scale          (rA) - value (single value array)
c                             for the attribute "scale_factor"

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      s  - string           [ character*(*) ]
c      iA - integer array    [ 4 byte, aka. "long" ]
c      rA - real array       [ 4 byte, aka. "float" ]


      subroutine write_nc_var_header(variable,type,dim_options,
     >                               var_n_dim,var_dim_ids,var_id,
     >                               extended,offset,scale)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'specs.inc'
      include 'name.inc'
      include 'err_codes.inc'

c     Input:
      integer type,dim_options,var_n_dim(dim_options),
     >        var_dim_ids(4,dim_options),extended
      real offset,scale
      character*(*) variable

c     Output:
      integer var_id

c     Local:
      integer size,long_name_size,nc_status,entry_no,dimension_option
      logical got_it
      character*20 temp_name,unit
      character*40 long_name

c     Function:
      integer str_len


c     Check 'type':
      if (type .ne. NF_SHORT .and. type .ne. NF_INT .and.
     >    type .ne. NF_FLOAT .and. type .ne. NF_DOUBLE) go to 999

c     Find variable no. on 'name.inc':
      size = str_len(variable)
      entry_no = 0
      got_it = .false.
      do while (entry_no .lt. no_var_name .and. .not. got_it)
         entry_no = entry_no + 1
         temp_name = var_name(entry_no)
         if (variable(1:size) .eq.
     >       temp_name(1:var_name_length(entry_no)))
     >      got_it = .true.
      end do
      if (.not. got_it) go to 998

c     --- START preparations ---

c     Set 'unit':
      if (variable(1:size) .eq. 'level') then
         unit = level_unit(leveltype)
         long_name = var_level_name(leveltype)
      else
         unit = var_unit(entry_no)
         long_name = var_long_name(entry_no)
      end if
      long_name_size = str_len(long_name)

      if (variable(1:size) .eq. 'lon' .or.
     >    variable(1:size) .eq. 'lat') then
         if (sphere_on .eq. 1) then
            if (variable(1:size) .eq. 'lon') then
               dimension_option = 1
            else
               dimension_option = 2
            end if
         else
            dimension_option = 6
         end if
      else
         dimension_option = var_dimension(entry_no)
      end if

c     --- END preparations ---


c     Explore existence:
      nc_status = NF_INQ_VARID(nc_id, variable(1:size), var_id)

c     Define?
      if (nc_status .eq. no_var_match) then
c        Define variable:
         nc_status = NF_DEF_VAR(nc_id, variable(1:size), type,
     >                          var_n_dim(dimension_option), 
     >                          var_dim_ids(1,dimension_option), var_id)
         if (nc_status .ne. NF_NOERR) call handle_err(nc_status,851)
      else if (nc_status .eq. NF_NOERR) then
         write(*,*)'  ... variable "',variable(1:size),'" previously ',
     >             'defined. Continuing execution.'
      else
         call handle_err(nc_status,852)
      end if


c     --- START include attributes if requested ---

      if (extended .eq. 1) then

         call nc_char_attrib(var_id,'long_name',long_name,variable)
         call nc_char_attrib(var_id,'units',unit,variable)
         call nc_float_attrib(var_id,'scale_factor',1,scale,variable)
         call nc_float_attrib(var_id,'add_offset',1,offset,variable)
         if (type .eq. NF_SHORT) then
            call nc_miss_val_i2(var_id, variable(1:size))
         else if (type .eq. NF_INT) then
            call nc_miss_val_i4(var_id, variable(1:size))
         else if (type .eq. NF_FLOAT) then
            call nc_miss_val_r(var_id, variable(1:size))
         else if (type .eq. NF_DOUBLE) then
            call nc_miss_val_d(var_id, variable(1:size))
         end if

      end if

c     --- END include attributes if requested ---


c     Add climatology headers if requested and proper:
      if (climatology_on .eq. 1 .and.
     >    var_id .ne. time_id .and. entry_no .gt. 3 .and.
     >    var_dim_ids(var_n_dim(dimension_option),dimension_option) .eq.
     >     t_id)
     >   call add_nc_var_climate(var_id,unit,entry_no,offset,scale,
     >                           dim_options,var_n_dim,var_dim_ids)


      write(*,*)' Included variable: ',variable(1:size),' / ',
     >                                 long_name(1:long_name_size)

      go to 1000

 998  write(*,*)
      write(*,*)'Error in the subroutine "write_nc_var_header".'
      write(*,*)' The variable name given: ',variable(1:size),
     >          ' is not implemented.'
      write(*,*)' (See the file "name.inc" for a list of implemented ',
     >          ' variables; as "var_name".)'
      write(*,*)'Terminating.'
      write(*,*)
      stop

 999  write(*,*)
      write(*,*)'Error in the subroutine "write_nc_var_header".'
      write(*,*)' Unknown type of variable: ',type
      write(*,*)' Known types: ',NF_SHORT, ' (NF_SHORT); ',
     >                           NF_INT,   ' (NF_INT)',
     >                           NF_FLOAT, ' (NF_FLOAT)',
     >                           NF_DOUBLE,' (NF_DOUBLE)'
      write(*,*)'Terminating.'
      write(*,*)
      stop

 1000 return
      end



c     ********************
c     General description:
c     *********************

c     The subroutine "write_nc_time_header" includes
c      * a new variable 'time' of type FLOAT on the file with
c          netCDF ID 'nc_id'
c      * a  string  attribute "long_name"
c      * a  string  attribute "units" (<unit>_since_<date>)
c      * a  string  attribute "origin"
c      * a  string  attribute "delta_t"
c      * a  string  attribute "method"
c      * an integer attribute "climate_dates_on" (a switch: on/off: 1/0)


c     ***********************
c     Input/output variables:
c     ***********************

c      dim_options    (i)  - no. of sets of different (x,y,z,t)-dimensions
c      var_n_dim      (iA) - no. of dimensions for the various
c                             'dim_options' dimensional combinations
c      var_dim_ids    (iA) - netCDF IDs for the various dimensional
c                             combinations

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      iA - integer array    [ 4 byte, aka. "long" ]


      subroutine write_nc_time_header(dim_options,var_n_dim,var_dim_ids)

      implicit none

      include 'netcdf.inc'
      include 'id.inc'
      include 'specs.inc'

c     Input:
      integer dim_options,var_n_dim(dim_options),
     >        var_dim_ids(4,dim_options)

c     Local:
      integer size
      character*19 date
      character*120 nc_string

c     Function:
      integer str_len


c     Define variable:
      call write_nc_var_header
     >        ('time',NF_FLOAT,dim_options,var_n_dim,var_dim_ids,
     >         time_id,0,0.,0.)

c     Add attributes:

c     "long_name":
      call nc_char_attrib(time_id, 'long_name', 'Time', 'time')

c     "units":
 101  format(i4.4, 5(a1,i2.2))
      write(date,101)reference_date(1),'-',reference_date(2),'-',
     >               reference_date(3),' ',reference_date(4),':',
     >               reference_date(5),':',reference_date(6)
      size = str_len(timeunit)
      nc_string = timeunit(1:size)//' since '// date
      call nc_char_attrib(time_id, 'units', nc_string, 'time')


c     "origin":
      call nc_char_attrib(time_id, 'origin', date, 'time')

c     "delta_t"
      write(date,101)timestep(1),'-',timestep(2),'-',
     >               timestep(3),' ',timestep(4),':',
     >               timestep(5),':',timestep(6)
      call nc_char_attrib(time_id, 'delta_t', date, 'time')

c     "method":
      call nc_char_attrib(time_id, 'method', timemethod, 'time')

c     "climate_dates_on"
      call nc_int_attrib(time_id, 'climate_dates_on',
     >                   1, climate_dates_on, 'time')


      return
      end
