
ccc   Subroutine on "lonlat2dxdy":
ccc    lonlat2dxdy      - converts gridded longitudes and latitudes to
ccc                       grid spacing arrays, assuming that input
ccc                       longitudes and latitudes represent the
ccc                       positions at the center of each grid box
ccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c     ********************
c     General description:
c     ********************

c     The subroutine "lonlat2dxdy" converts gridded longitudes and
c     latitudes to grid spacing arrays. It is assumed that the
c     input longitudes and latitudes represent the position at the 
c     center of each grid box, and that the earth is a perfect sphere
c     with a radius of 6371 km. The output grid spacing applies to the
c     grid boxes (and not the distance between box centers), i.e.,:

c          .......   
c          .     .   |
c          .  x  .   | dy
c          .     .   |
c          .......   |

c          _______
c            dx

c     Since the grid boxes are usually not squares, dx and dy should
c     represent the distance across the center of the box. The
c     distances at the edges x=1, x=nx, y=1, y=ny are set by
c     extrapolation.


c     ***********************
c     Input/output variables:
c     ***********************

c      nx, ny         (i)  - horizontal dimensions of the fields
c      lon            (rA) - longitudes of grid box centers
c      lat            (rA) - latitudes  of grid box centers
c      dx             (rA) - length of the grid box along the x axis
c      dy             (rA) - length of the grid box along the y axis

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      rA - real array       [ 4 byte, aka. "float" ]


      subroutine lonlat2dxdy(nx,ny,lon,lat,dx,dy)

      implicit none

      real earthradius
      parameter (earthradius = 6371000.)

c     Input:
      integer nx,ny
      real lon(nx,ny),lat(nx,ny)

c     Output:
      real dx(nx,ny),dy(nx,ny)

c     Local:
      integer x,y
      real pi,lat_distance,factor,length,ds,trans_factor,radians


c     Find 'lat_distance', distance that corresponds to 1 deg. latitude:
      pi = 4.*atan(1.)
c      call earthr(earthradius)
      lat_distance = 2.*pi*earthradius/360.

c     Transformation factor:
      trans_factor = pi/180.

c     Set 'dx(,)':

      do y = 2,ny-1

         radians = trans_factor* 0.5*(lat(1,y) + lat(2,y))
         factor = cos(radians)
         length = sqrt( ((lon(2,y) - lon(1,y))*factor)**2 +
     >                  (lat(2,y) - lat(1,y))**2 )
         length = lat_distance*length

         do x = 2,nx-1
            ds = length
            length = sqrt( ((lon(x+1,y) - lon(x,y))*factor)**2 +
     >                     (lat(x+1,y) - lat(x,y))**2 )
            length = lat_distance*length
            dx(x,y) = 0.5*(ds + length)
         end do

      end do

      do y = 2,ny-1
         dx(1,y) = 2.*dx(2,y) - dx(3,y)
         dx(nx,y) = 2.*dx(nx-1,y) - dx(nx-2,y)
      end do

      do x = 2,nx-1
         dx(x,1) = 2.*dx(x,2) - dx(x,3)
         dx(x,ny) = 2.*dx(x,ny-1) - dx(x,ny-2)
      end do

      dx(1,1) = dx(2,1) - 0.5*dx(3,1) + dx(1,2) - 0.5*dx(1,3)
      dx(nx,1) = dx(nx-1,1) - 0.5*dx(nx-2,1) + dx(nx,2) - 0.5*dx(nx,3)
      dx(1,ny) = dx(2,ny) - 0.5*dx(3,ny) + dx(1,ny-1) - 0.5*dx(1,ny-2)
      dx(nx,ny) = dx(nx-1,ny) - 0.5*dx(nx-2,ny) +
     >            dx(nx,ny-1) - 0.5*dx(nx,ny-2)

c     Set 'dy(,)':

      do x = 2,nx-1

         radians = trans_factor* 0.5*(lat(x,1) + lat(x,2))
         factor = cos(radians)
         length = sqrt( ((lon(x,2) - lon(x,1))*factor)**2 +
     >                  (lat(x,2) - lat(x,1))**2 )
         length = lat_distance*length

         do y = 2,ny-1
            ds = length
            length = sqrt( ((lon(x,y+1) - lon(x,y))*factor)**2 +
     >                     (lat(x,y+1) - lat(x,y))**2 )
            length = lat_distance*length
            dy(x,y) = 0.5*(ds + length)
         end do

      end do

      do y = 2,ny-1
         dy(1,y) = 2.*dy(2,y) - dy(3,y)
         dy(nx,y) = 2.*dy(nx-1,y) - dy(nx-2,y)
      end do

      do x = 2,nx-1
         dy(x,1) = 2.*dy(x,2) - dy(x,3)
         dy(x,ny) = 2.*dy(x,ny-1) - dy(x,ny-2)
      end do

      dy(1,1) = dy(2,1) - 0.5*dy(3,1) + dy(1,2) - 0.5*dy(1,3)
      dy(nx,1) = dy(nx-1,1) - 0.5*dy(nx-2,1) + dy(nx,2) - 0.5*dy(nx,3)
      dy(1,ny) = dy(2,ny) - 0.5*dy(3,ny) + dy(1,ny-1) - 0.5*dy(1,ny-2)
      dy(nx,ny) = dy(nx-1,ny) - 0.5*dy(nx-2,ny) +
     >            dy(nx,ny-1) - 0.5*dy(nx,ny-2)


      return
      end
