
ccc   Subroutine on "short2short.f":
ccc    short2short_1d   - to be included at a later stage
ccc    short2short_2d   - converts one 2D integer*2 field to another,
ccc                       due to changes in scaling, offset,
ccc                       missing value and/or fill value
ccc    short2short_3d   - to be included at a later stage
ccc
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc


c     ********************
c     General description:
c     ********************

c     The subroutine "short2short_2d" converts an original 2D integer*2
c     field, due to changes in one or more of
c      - scaling
c      - offset
c      - missing value
c      - fill value
c     The present implementation does not support output of fill values,
c     so input missing values and fill values will all be stored as
c     output missing values.
c     The subroutine terminates if a proper output value is:
c      - either converted to a value outside integer*2 range
c        (-32768,32767 inclusive)
c      - or converted to the output missing value


c     ***********
c     Conversion:
c     ***********

c       real value = org_short*org_scale + org_offset
c                  = new_short*scale     + offset
c     so,
c                   org_short*org_scale + org_offset - offset
c       new_short = -----------------------------------------
c                                 scale


c     ***********************
c     Input/output variables:
c     ***********************

c      nx, ny         (i)  - horizontal dimensions of 'field'
c      field          (i2A)- horizontal field, input and output
c      org_offset     (r)  - offset used in 'field' upon input
c      org_scale      (r)  - scale factor used in 'field' upon input
c      org_missval    (i2) - missing value used in 'field' upon input
c      org_fillval    (i2) - fill value used in 'field' upon input
c      offset         (r)  - offset used in 'field' upon output
c      scale          (r)  - scale factor used in 'field' upon output

c     Above:
c      i  - integer variable [ 4 byte, aka. "long" ]
c      i2 - integer variable [ 2 byte, aka. "short" ]
c      r  - real variable    [ 4 byte, aka. "float" ]
c      i2A- integer*2 array  [ 2 byte, aka. "short" ]
c      rA - real array       [ 4 byte, aka. "float" ]


      subroutine short2short_2d(nx,ny,field,
     >                          org_offset,org_scale,
     >                          org_missval,org_fillval,
     >                          offset,scale)

      implicit none

      include 'missing_values.inc'

c     Input:
      integer*2 org_missval,org_fillval
      integer nx,ny
      real org_offset,org_scale,offset,scale

c     Input/output:
      integer*2 field(nx,ny)

c     Local:
      integer x,y,value
      real add_offset


c     Set 'add_offset':
      add_offset = org_offset - offset


c     --- START Conversion ---

      if (add_offset .ne. 0 .or. org_missval .ne. miss_val_i2(1) .or.
     >                           org_fillval .ne. miss_val_i2(1)) then

         do y = 1,ny
            do x = 1,nx
               if (field(x,y) .eq. org_missval .or.
     >             field(x,y) .eq. org_fillval) then
                  field(x,y) = miss_val_i2(1)
               else
                  value = nint( (float(field(x,y))*org_scale +
     >                           add_offset) / scale )
                  if (value .eq. miss_val_i2(1)) then
                     go to 999
                  else if (value .gt. 32767 .or. value .lt. -32768) then
                     go to 998
                  else
                     field(x,y) = value
                  end if
               end if
            end do
         end do

      end if

c     --- END conversion ---


      go to 1000


 998  write(*,*)
      write(*,*)'Error in the subroutine "short2short_2d".'
      write(*,*)' The new value at node (x, y) =',x,y
      write(*,*)'  attains an out-of-range value:',value
      write(*,*)' Terminates.'
      write(*,*)
      stop

 999  write(*,*)
      write(*,*)'Error in the subroutine "short2short_2d".'
      write(*,*)' The new value at node (x, y) =',x,y
      write(*,*)'  was originally non-missing (or wet),'
      write(*,*)'  but attains the missing value upon conversion.'
      write(*,*)' Terminates.'
      write(*,*)
      stop

 1000 return
      end

